-- **************************************************************************
--  This is Analog ADF4113 Pll IC control
--				(C)Copyright 2007,08,07 by Y nishimura
-- **************************************************************************
LIBRARY ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity PLLIC is
	port
		(
			Hclk		: in  std_logic;	-- Master clock
			Reset		: in  std_logic;	-- Master Reset
			Cgate		: in  std_logic;	-- PLL set Trigger
			Plllock		: in  std_logic;	-- PLL lock signal
			SCLK		: out std_logic;	-- PLL Serial clock
			DATA		: out std_logic;	-- PLL data
			BUSY		: out std_logic;	-- Data busy
			LE			: out std_logic;	-- PLL Latch enable
			Intp		: out std_logic;	-- PLL intial set mode
			Clk10MHz	: out std_logic;	-- 10MHz out
			Pll65trig	: out std_logic		-- PLL 250MHz Trigger enable
		);
end PLLIC;

architecture norm of PLLIC is
	signal	Sdata,WaitCnt		: std_logic_vector(23 downto 0);
	signal	Stat				: std_logic_vector(7 downto 0);
	signal	TrigSta				: std_logic_vector(1 downto 0);
	signal	Idle,Load,Lena,Ckk	: std_logic;
	signal	Rclk,DetTime,Itrig	: std_logic;
	signal	Initial,Pll65tr				: std_logic;

begin
	SCLK <= Rclk; DATA <= Sdata(23); LE <= Lena; BUSY <= Idle;
	Intp <= Initial; Pll65trig <= Pll65tr;
-------------------------------------------------------------------------------
--		Sequence counter
-------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' then
			Stat <= "00000000"; Idle <= '0'; Ckk <= '0';
			Rclk <= '0';
		elsif rising_edge(Hclk) then
			if Idle = '0'				then Idle <= Itrig;
			else
				if Stat = "11111111"	then Idle <= '0';
										else Idle <= '1';
				end if;
			end if;
			if Idle = '1'			then Stat <= Stat + 1;
									else Stat <= "00000000";
			end if;
			if Stat(5 downto 0) = "000001"	then Ckk <= '1';
			else
				if Stat(5 downto 0) = "110001"	then Ckk <= '0';
												else Ckk <= Ckk;
				end if;
			end if;
			if Ckk = '0'			then Rclk <= '0';
									else Rclk <= not Rclk;
			end if;
		end if;
	end process;

-------------------------------------------------------------------------------
--		Load timing
-------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' then Load <= '0'; Lena <= '0';
		elsif rising_edge(Hclk) then
			if Idle = '1'		then
				if Stat(5 downto 0) = "000001"	then Load <= '1';
												else Load <= '0';
				end if;
			else Load <= '0';
			end if;
			if Idle = '1'		then
				if Stat(5 downto 0) = "110010"	then Lena <= '1';
				else
					if Stat(0) = '1'			then Lena <= '0';
												else Lena <= Lena;
					end if;
				end if;
			else Lena <= '0';
			end if;
		end if;
	end process;
	
-------------------------------------------------------------------------------
--		Shift register
--				F=(P*B+A)*10MHz/R    F=1GHz/8 P=8 B=250 A=0 R=20
--						Compare Frequency = 10MHz/20=500KHz
-------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' then Sdata <= "000000000000000000000000";
		elsif rising_edge(Hclk) then
			if Load = '1'	then
				case Stat(7 downto 6) is
					when "00" => if Initial = '1' then Sdata <= "000010000011111010010011";
												  else Sdata <= "000010000011111010010010";
								 end if;
					when "01" => Sdata <= "000100100000000001010000";  -- 20=0x14
					when "10" => Sdata <= "001000001111101000000001";  -- 1GHz=0xFA A=0
					when "11" => Sdata <= "000100100000000001010000";  -- 20=0x14
					when others => Sdata <= "000000000000000000000000";
				end case;
			else
				if Stat(0) = '1'	then
					Sdata(23 downto 1) <= Sdata(22 downto 0);
					Sdata(0) <= '0';
				else Sdata <= Sdata;
				end if;
			end if;
		end if;
	end process;

---------------------------------------------------------------------------------
--		Phase Lock check interval timmer 1.67sec interval
---------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' then
			WaitCnt <= "000000000000000000000000";
			DetTime <= '0';
		elsif rising_edge(Hclk) then
			WaitCnt <= WaitCnt + 1;
			if WaitCnt = "010000000000000000000000" then
--			if WaitCnt(10 downto 0) = "00000010000" then
				 DetTime <= '1';
			else DetTime <= '0';
			end if;
		end if;
	end process;
	Clk10MHz <= WaitCnt(1);
	
-----------------------------------------------------------------------------------
--		Trigger status
-----------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' then	TrigSta <= "00"; Initial <= '0';
		elsif rising_edge(Hclk) then
			case TrigSta is
				when "00" => if DetTime = '1'	then TrigSta <= "01";
												else TrigSta <= "00";
							 end if;
							 Initial <= '0';
				when "01" => TrigSta <= "11"; Initial <= '1';
				when "10" => TrigSta <= "11"; Initial <= '0';
				when "11" => if DetTime = '1'	then TrigSta <= "10";
												else TrigSta <= "11";
							 end if;
							 Initial <= Initial;
				when others => TrigSta <= "00"; Initial <= '0';
			end case;
		end if;
	end process;

------------------------------------------------------------------------------------
--		Trigger Sequence
------------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' then 
			Pll65tr <= '0'; Itrig <= '0';
		elsif rising_edge(Hclk) then
			if Itrig = '1'			then Pll65tr <= '1';
			else
				if Cgate = '1'		then Pll65tr <= '0';
									else Pll65tr <= Pll65tr;
				end if;
			end if;
			if TrigSta = "01"	 	then Itrig <= '1';
			else
				if TrigSta = "10"	then Itrig <= not PllLock;
									else Itrig <= '0';
				end if;
			end if;
		end if;
	end process;

end norm;

